/**
  @file dbus-handler.h

  Copyright (C) 2004 Nokia Corporation. All rights reserved.

  @author Janne Ylalehto <janne.ylalehto@nokia.com>

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef _DBUS_HANDLER_H_
#define _DBUS_HANDLER_H_

#include <sys/types.h>
#include <sys/socket.h>
#include <linux/types.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>
#include <linux/if_ether.h>
#define DBUS_API_SUBJECT_TO_CHANGE
#include <dbus/dbus.h>
#include <wlancond.h>

#include "libnl-handler.h"
#include "common.h"

/** scan callback data */
typedef struct scan_cb_data {
	char ssid[WLANCOND_MAX_SSID_SIZE+1];
	guint32 ssid_len;
	guint32 count;
} scan_cb_data;

enum {
	SCAN_NORMAL = 0,
	SCAN_SPLIT
};

extern struct wlan_status_t wlan_status;

/** Bind functions to corresponding D-Bus messages
 * @param connection D-Bus connection
 */
void init_dbus_handlers(DBusConnection *connection);

/** Free memory allocated to handlers
 * @param connection D-Bus connection
 */
void destroy_dbus_handlers(DBusConnection *connection);

int socket_open(void);
gboolean ask_scan_results(void);
void set_wlan_state(int new_state, int send_signal, gboolean delay_shutdown);
guint get_wlan_state(void);
void set_scan_state(guint new_state);
guint get_scan_state(void);
int init_dbus_handler(void);
int clean_dbus_handler(void);
void clear_wpa_mode(void);
int wpa_ie_push(char* ap_mac_addr);
gboolean get_wpa_mode(void);
int set_interface_state(int sock, int dir, short flags);
void clean_roam_cache(void);
guint get_mode(void);
gboolean set_real_power_state(guint new_state, int sock);
int scan(gchar *ssid, int ssid_len, gboolean add_timer);
void update_own_ie(unsigned char* wpa_ie, guint wpa_ie_len);
gboolean set_power_state(guint state);
guint get_encryption_info(void);
void remove_connect_timer(void);
int disassociate_eap(void);
int check_pmksa_cache(unsigned char* own_mac, int own_mac_len,
		unsigned char* bssid, int bssid_len,
		uint32_t authentication_type,
		uint32_t pairwise_key_cipher_suite,
		uint32_t group_key_cipher_suite,
		int *status);
void set_call_type(const char *type);
int context_parser(DBusMessageIter *actit);
int wpa_mic_failure_event(dbus_bool_t key_type, dbus_bool_t is_fatal);
void save_device_interface(void);
gboolean remove_from_pmksa_cache(unsigned char* mac);
int find_pmkid_from_pmk_cache(unsigned char* mac, unsigned char** pmkid);
DBusHandlerResult wlancond_req_handler(DBusConnection *connection,
		DBusMessage *message, void *user_data);

int associate(struct scan_results_t *scan_results);
gboolean remove_from_roam_cache(roam_cache_key_t* const key);
struct scan_results_t* find_connection(
		GSList* scan_list, struct connect_params_t *conn,
		gboolean update_roam_cache);
void remove_roam_scan_timer(void);
void remove_scan_timer(void);
int set_bssid(unsigned char *bssid);
int set_essid(char* essid, int essid_len);
void clear_essid_and_bssid(void);
int find_connection_and_associate(GSList *scan_results,
				  gboolean update_roam_cache,
				  gboolean create_new_adhoc,
				  gboolean autoconnect);
int scan_results_ioctl(int ifindex, GSList** scan_results_save, char* ssid, 
		       guint ssid_len);
void set_wlan_signal(gboolean high_or_low);
gboolean decrease_signal_in_roam_cache(roam_cache_key_t* const key);
int mlme_command(guchar* addr, guint16 cmd, guint16 reason_code);
void schedule_scan(guint seconds);
gboolean scan_retries_left(void);
void connected_signal(char* bssid);
gint run_calibration(void);
void kill_supplicant(void);
gboolean get_inactivity_status(void);
gboolean get_setting_bool(const gchar* path, gboolean *error);
gboolean can_adjust_psm(void);
struct scan_results_t* copy_scan_results(struct scan_results_t *results);
gboolean set_tx_power(guint power);

#define CLEAR 1
#define SET   2

#define ETOOMANYREGISTRARS -500
#define ETOOWEAKAP         -2
#define ENEEDEXTRASCAN     -3
#define ESUPPLICANT        -5
#define EWRONGSSID         -7

#endif /* _DBUS_HANDLER_H_ */
