/**
  @file libnl-handler.h

  Copyright (C) 2010 Nokia Corporation. All rights reserved.

  @author Usman Rahim <usman.rahim@nokia.com>

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef _LIBNL_HANDLER_H_
#define _LIBNL_HANDLER_H_

#include <netlink/genl/genl.h>
#include <netlink/genl/family.h>
#include <netlink/genl/ctrl.h>
#include <linux/nl80211.h>

#include "common.h"

/* Cipher suites */
#define WLANCOND_CIPHER_SUITE_NONE    0x000FAC00
#define WLANCOND_CIPHER_SUITE_WEP40   0x000FAC01
#define WLANCOND_CIPHER_SUITE_TKIP    0x000FAC02
#define WLANCOND_CIPHER_SUITE_CCMP    0x000FAC04
#define WLANCOND_CIPHER_SUITE_WEP104  0x000FAC05

/* Key management suites */
#define WLANCOND_KEY_MGMT_802_1X      0x000FAC01
#define WLANCOND_KEY_MGMT_PSK         0x000FAC02

/* Defines for connection quality monitoring */
#define WLANCOND_CQM_THRESHOLD_ACTIVE -70
#define WLANCOND_CQM_THRESHOLD_IDLE   -80
#define WLANCOND_CQM_HYSTERESIS         3

struct get_family_t {
    const char *group_name;
    int id;
};

int init_nl80211(void);
int nl_send_recv_msgs(struct nl_msg *msg, int (*valid_handler)(struct nl_msg *, void *), void *valid_data);
void nl80211_cleanup(void);
int nl80211_connect(struct scan_results_t *scan_results, struct wlan_status_t *wlan_status);
int nl80211_leave_adhoc(guint32 ifindex);
int nl80211_set_op_mode(guint32 ifindex, guint32 mode);
int nl80211_mlme_command(guchar* addr, guint16 cmd, guint16 reason_code, guint32 ifindex);
int nl80211_get_signal_info(guint8 *power, struct wlan_status_t *wlan_status);
int nl80211_set_power_save(gboolean new_state, guint32 ifindex);
int nl80211_scan(wlan_status_t* const wlan_status, int ssid_len, char* const ssid);
int nl80211_send_get_scan_results(wlan_status_t* const wlan_status, GSList** const scan_results);
void nl80211_scan_ready(void);
int nl80211_set_cqm(guint32 threshold, guint32 hyst);
int nl80211_set_tx_power(int tx_power, guint32 ifindex);
int nl80211_get_phy_info(guint32 ifindex);
int nl80211_set_channel(guint32 ifindex, guint32 channel);

#endif

