/**
  @file dbus-helper.h

  Copyright (C) 2004-2010 Nokia Corporation. All rights reserved.

  @author Janne Yllehto <janne.ylalehto@nokia.com>

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#ifndef _DBUS_HELPER_H_
#define _DBUS_HELPER_H_

#ifndef DBUS_API_SUBJECT_TO_CHANGE
# define DBUS_API_SUBJECT_TO_CHANGE
# include <dbus/dbus.h>
#endif

#define RULE_MCE_MODE     (1<<0)
#define RULE_MCE_ACTIVITY (1<<1)
#define RULE_ICD          (1<<2)
#define RULE_EAP          (1<<3)
#define RULE_CSD          (1<<4)
#define RULE_CONTEXT      (1<<5)
#define RULE_GPRS         (1<<6)

enum add_or_remove_t {
	ADD = 1,
	REMOVE
};

void append_dbus_args(DBusMessage *message, int first_arg_type, ...);

void send_and_unref(DBusConnection *connection, DBusMessage *message);

void send_invalid_args(DBusConnection *connection, DBusMessage *message);

DBusMessage *new_dbus_signal(const char *path,
		const char *interface,
		const char *name,
		const char *destination);

DBusMessage *new_dbus_method_return(DBusMessage *message);

DBusMessage *new_dbus_error(DBusMessage *message, const char *name);

gint get_device_mode(DBusConnection *connection, dbus_uint32_t *radio_mode);
dbus_bool_t get_activity(void);
guint get_call_state(void);
char* check_country_code(void);
gboolean add_mode_listener(DBusConnection *connection);
gboolean add_cover_listener(DBusConnection *connection,
		void (*cover_cb)(void));
gboolean add_icd_listener(DBusConnection *connection);
void remove_icd_listener(DBusConnection *connection);
gboolean add_csd_listener(DBusConnection *connection);
gboolean add_eap_listener(DBusConnection *connection);
void remove_eap_listener(DBusConnection *connection);
#ifdef ACTIVITY_CHECK
gboolean add_activity_listener(DBusConnection *connection);
void remove_activity_listener(DBusConnection *connection);
#endif
gboolean add_context_listener(DBusConnection *connection);
void remove_context_listener(DBusConnection *connection);
gboolean add_gprs_listener(DBusConnection *connection);
void remove_gprs_listener(DBusConnection *connection);

#endif /* _DBUD_HELPER_H_ */
